@@ -49,6 +49,7 @@ import org.apache.commons.scxml.model.If;
 import org.apache.commons.scxml.model.Initial;
 import org.apache.commons.scxml.model.Invoke;
 import org.apache.commons.scxml.model.Log;
+import org.apache.commons.scxml.model.NamespacePrefixesHolder;
 import org.apache.commons.scxml.model.OnEntry;
 import org.apache.commons.scxml.model.OnExit;
 import org.apache.commons.scxml.model.Parallel;
@@ -62,11 +63,22 @@ import org.apache.commons.scxml.model.Var;
 import org.w3c.dom.Node;
 
 /**
- * Utility class for serializing the Commons SCXML Java object
+ * <p>Utility class for serializing the Commons SCXML Java object
  * model. Class uses the visitor pattern to trace through the
  * object heirarchy. Used primarily for testing, debugging and
- * visual verification.
+ * visual verification.</p>
  *
+ * <b>NOTE:</b> This serializer makes the following assumptions about the
+ * original SCXML document(s) parsed to create the object model:
+ * <ul>
+ *  <li>The default document namespace is the SCXML namespace:
+ *      <i>http://www.w3.org/2005/07/scxml</i></li>
+ *  <li>The Commons SCXML namespace
+ *      ( <i>http://commons.apache.org/scxml</i> ), if needed, uses the
+ *      &quot;<i>cs</i>&quot; prefix</li>
+ *  <li>All namespace prefixes needed throughout the document are
+ *      declared on the document root element (&lt;scxml&gt;)</li>
+ * </ul>
  */
 public class SCXMLSerializer {
 
@@ -74,6 +86,12 @@ public class SCXMLSerializer {
     private static final String INDENT = " ";
     /** The JAXP transformer. */
     private static final Transformer XFORMER = getTransformer();
+    /** The SCXML namespace. */
+    private static final String NAMESPACE_SCXML =
+        "http://www.w3.org/2005/07/scxml";
+    /** The Commons SCXML namespace. */
+    private static final String NAMESPACE_COMMONS_SCXML =
+        "http://commons.apache.org/scxml";
 
     /**
      * Serialize this SCXML object (primarily for debugging).
@@ -85,9 +103,10 @@ public class SCXMLSerializer {
     public static String serialize(final SCXML scxml) {
         StringBuffer b =
             new StringBuffer("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n").
-                append("<scxml xmlns=\"").append(scxml.getXmlns()).
-                append("\" version=\"").append(scxml.getVersion()).
-                append("\" initialstate=\"").append(scxml.getInitialstate()).
+                append("<scxml xmlns=\"").append(NAMESPACE_SCXML).
+                append("\"").append(serializeNamespaceDeclarations(scxml)).
+                append(" version=\"").append(scxml.getVersion()).
+                append("\" initial=\"").append(scxml.getInitial()).
                 append("\">\n");
         if (XFORMER == null) {
             org.apache.commons.logging.Log log = LogFactory.
@@ -96,14 +115,20 @@ public class SCXMLSerializer {
                 + " the document will be skipped since a suitable"
                 + " JAXP Transformer could not be instantiated.");
         }
+        b.append(INDENT).append("<!-- http://commons.apache.org/scxml -->\n");
         Datamodel dm = scxml.getDatamodel();
         if (dm != null) {
             serializeDatamodel(b, dm, INDENT);
         }
-        Map s = scxml.getStates();
-        Iterator i = s.keySet().iterator();
+        Map c = scxml.getChildren();
+        Iterator i = c.keySet().iterator();
         while (i.hasNext()) {
-            serializeState(b, (State) s.get(i.next()), INDENT);
+            TransitionTarget tt = (TransitionTarget) c.get(i.next());
+            if (tt instanceof State) {
+                serializeState(b, (State) tt, INDENT);
+            } else {
+                serializeParallel(b, (Parallel) tt, INDENT);
+            }
         }
         b.append("</scxml>\n");
         return b.toString();
@@ -120,7 +145,7 @@ public class SCXMLSerializer {
             final State s, final String indent) {
         b.append(indent).append("<state");
         serializeTransitionTargetAttributes(b, s);
-        boolean f = s.getIsFinal();
+        boolean f = s.isFinal();
         if (f) {
             b.append(" final=\"true\"");
         }
@@ -138,16 +163,11 @@ public class SCXMLSerializer {
             serializeDatamodel(b, dm, indent + INDENT);
         }
         serializeOnEntry(b, s, indent + INDENT);
-        Map t = s.getTransitions();
-        Iterator i = t.keySet().iterator();
-        while (i.hasNext()) {
-            List et = (List) t.get(i.next());
-            for (int len = 0; len < et.size(); len++) {
-                serializeTransition(b, (Transition) et.get(len), indent
-                    + INDENT);
-            }
+        List t = s.getTransitionsList();
+        for (int i = 0; i < t.size(); i++) {
+            serializeTransition(b, (Transition) t.get(i), indent + INDENT);
         }
-        Parallel p = s.getParallel();
+        Parallel p = s.getParallel(); //TODO: Remove in v1.0
         Invoke inv = s.getInvoke();
         if (p != null) {
             serializeParallel(b, p, indent + INDENT);
@@ -157,8 +177,12 @@ public class SCXMLSerializer {
             Map c = s.getChildren();
             Iterator j = c.keySet().iterator();
             while (j.hasNext()) {
-                State cs = (State) c.get(j.next());
-                serializeState(b, cs, indent + INDENT);
+                TransitionTarget tt = (TransitionTarget) c.get(j.next());
+                if (tt instanceof State) {
+                    serializeState(b, (State) tt, indent + INDENT);
+                } else if (tt instanceof Parallel) {
+                    serializeParallel(b, (Parallel) tt, indent + INDENT);
+                }
             }
         }
         serializeOnExit(b, s, indent + INDENT);
@@ -178,7 +202,7 @@ public class SCXMLSerializer {
         serializeTransitionTargetAttributes(b, p);
         b.append(">\n");
         serializeOnEntry(b, p, indent + INDENT);
-        Set s = p.getStates();
+        Set s = p.getChildren();
         Iterator i = s.iterator();
         while (i.hasNext()) {
             serializeState(b, (State) i.next(), indent + INDENT);
@@ -215,7 +239,7 @@ public class SCXMLSerializer {
             Param p = (Param) iter.next();
             b.append(indent).append(INDENT).append("<param name=\"").
                 append(p.getName()).append("\" expr=\"").
-                append(p.getExpr()).append("\"/>\n");
+                append(SCXMLHelper.escapeXML(p.getExpr())).append("\"/>\n");
         }
         Finalize f = i.getFinalize();
         if (f != null) {
@@ -277,10 +301,21 @@ public class SCXMLSerializer {
      */
     public static void serializeTransition(final StringBuffer b,
             final Transition t, final String indent) {
-        b.append(indent).append("<transition event=\"").append(t.getEvent())
-            .append("\" cond=\"").append(t.getCond()).append("\">\n");
+        b.append(indent).append("<transition");
+        if (!SCXMLHelper.isStringEmpty(t.getEvent())) {
+            b.append(" event=\"").append(t.getEvent()).append("\"");
+        }
+        if (!SCXMLHelper.isStringEmpty(t.getCond())) {
+            b.append(" cond=\"").append(SCXMLHelper.escapeXML(t.getCond())).
+                append("\"");
+        }
+        boolean next = !SCXMLHelper.isStringEmpty(t.getNext());
+        if (next) {
+            b.append(" target=\"" + t.getNext() + "\"");
+        }
+        b.append(">\n");
         boolean exit = serializeActions(b, t.getActions(), indent + INDENT);
-        if (!exit) {
+        if (!next && !exit) {
             serializeTarget(b, t, indent + INDENT);
         }
         b.append(indent).append("</transition>\n");
@@ -293,21 +328,18 @@ public class SCXMLSerializer {
      * @param b The buffer to append the serialization to
      * @param t The Transition whose Target needs to be serialized
      * @param indent The indent for this XML element
+     *
+     * @deprecated Inline &lt;target&gt; element has been deprecated
+     *             in the SCXML WD
      */
     public static void serializeTarget(final StringBuffer b,
             final Transition t, final String indent) {
-        b.append(indent).append("<target");
-        String n = t.getNext();
-        if (n != null) {
-            b.append(" next=\"" + n + "\">\n");
-        } else {
-            b.append(">\n");
-            if (t.getTarget() != null) {
-                // The inline transition target can only be a state
-                serializeState(b, (State) t.getTarget(), indent + INDENT);
-            }
+        if (t.getTarget() != null) {
+            b.append(indent).append("<target>");
+            // The inline transition target can only be a state
+            serializeState(b, (State) t.getTarget(), indent + INDENT);
+            b.append(indent).append("</target>");
         }
-        b.append(indent).append("</target>\n");
     }
 
     /**
@@ -346,9 +378,10 @@ public class SCXMLSerializer {
                     }
                     b.append(indent).append(INDENT).append(out.toString());
                 } else {
-                    b.append(indent).append(INDENT).append("<data name=\"").
-                        append(datum.getName()).append("\" expr=\"").
-                        append(datum.getExpr()).append("\" />\n");
+                    b.append(indent).append(INDENT).append("<data id=\"").
+                        append(datum.getId()).append("\" expr=\"").
+                        append(SCXMLHelper.escapeXML(datum.getExpr())).
+                        append("\" />\n");
                 }
             }
             b.append(indent).append("</datamodel>\n");
@@ -408,9 +441,10 @@ public class SCXMLSerializer {
             Action a = (Action) i.next();
             if (a instanceof Var) {
                 Var v = (Var) a;
-                b.append(indent).append("<var name=\"").append(v.getName())
-                        .append("\" expr=\"").append(v.getExpr()).append(
-                                "\"/>\n");
+                b.append(indent).append("<cs:var name=\"").append(v.getName())
+                    .append("\" expr=\"")
+                    .append(SCXMLHelper.escapeXML(v.getExpr()))
+                    .append("\"/>\n");
             } else if (a instanceof Assign) {
                 Assign asn = (Assign) a;
                 b.append(indent).append("<assign");
@@ -419,11 +453,13 @@ public class SCXMLSerializer {
                     if (!SCXMLHelper.isStringEmpty(asn.getSrc())) {
                         b.append("\" src=\"").append(asn.getSrc());
                     } else {
-                        b.append("\" expr=\"").append(asn.getExpr());
+                        b.append("\" expr=\"").
+                            append(SCXMLHelper.escapeXML(asn.getExpr()));
                     }
                 } else {
                     b.append(" name=\"").append(asn.getName()).
-                        append("\" expr=\"").append(asn.getExpr());
+                        append("\" expr=\"").
+                        append(SCXMLHelper.escapeXML(asn.getExpr()));
                 }
                 b.append("\"/>\n");
             } else if (a instanceof Send) {
@@ -434,12 +470,13 @@ public class SCXMLSerializer {
                     .append(c.getSendid()).append("\"/>\n");
             } else if (a instanceof Log) {
                 Log lg = (Log) a;
-                b.append(indent).append("<log expr=\"").append(lg.getExpr())
-                        .append("\"/>\n");
+                b.append(indent).append("<log expr=\"").
+                    append(SCXMLHelper.escapeXML(lg.getExpr())).
+                    append("\"/>\n");
             } else if (a instanceof Exit) {
                 Exit e = (Exit) a;
-                b.append(indent).append("<exit");
-                String expr = e.getExpr();
+                b.append(indent).append("<cs:exit");
+                String expr = SCXMLHelper.escapeXML(e.getExpr());
                 String nl = e.getNamelist();
                 if (expr != null) {
                     b.append(" expr=\"" + expr + "\"");
@@ -457,7 +494,8 @@ public class SCXMLSerializer {
             } else if (a instanceof ElseIf) {
                 ElseIf eif = (ElseIf) a;
                 b.append(indent).append("<elseif cond=\"")
-                        .append(eif.getCond()).append("\" />\n");
+                    .append(SCXMLHelper.escapeXML(eif.getCond()))
+                    .append("\" />\n");
             }
         }
         return exit;
@@ -472,16 +510,31 @@ public class SCXMLSerializer {
      */
     public static void serializeSend(final StringBuffer b,
             final Send send, final String indent) {
-        b.append(indent).append("<send sendid=\"")
-            .append(send.getSendid()).append("\" target=\"")
-            .append(send.getTarget()).append("\" targetType=\"")
-            .append(send.getTargettype()).append("\" namelist=\"")
-            .append(send.getNamelist()).append("\" delay=\"")
-            .append(send.getDelay()).append("\" events=\"")
-            .append(send.getEvent()).append("\" hints=\"")
-            .append(send.getHints()).append("\">\n")
-            .append(getBodyContent(send))
-            .append(indent).append("</send>\n");
+        b.append(indent).append("<send");
+        if (send.getSendid() != null) {
+            b.append(" sendid=\"").append(send.getSendid()).append("\"");
+        }
+        if (send.getTarget() != null) {
+            b.append(" target=\"").append(send.getTarget()).append("\"");
+        }
+        if (send.getTargettype() != null) {
+            b.append(" targetType=\"").append(send.getTargettype()).append("\"");
+        }
+        if (send.getNamelist() != null) {
+            b.append(" namelist=\"").append(send.getNamelist()).append("\"");
+        }
+        if (send.getDelay() != null) {
+            b.append(" delay=\"").append(send.getDelay()).append("\"");
+        }
+        if (send.getEvent() != null) {
+            b.append(" event=\"").append(send.getEvent()).append("\"");
+        }
+        if (send.getHints() != null) {
+            b.append(" hints=\"").append(send.getHints()).append("\"");
+        }
+        b.append(">\n");
+        b.append(getBodyContent(send));
+        b.append(indent).append("</send>\n");
     }
 
     /**
@@ -524,8 +577,8 @@ public class SCXMLSerializer {
      */
     public static void serializeIf(final StringBuffer b,
             final If iff, final String indent) {
-        b.append(indent).append("<if cond=\"").append(iff.getCond()).append(
-                "\">\n");
+        b.append(indent).append("<if cond=\"").append(SCXMLHelper.
+            escapeXML(iff.getCond())).append("\">\n");
         serializeActions(b, iff.getActions(), indent + INDENT);
         b.append(indent).append("</if>\n");
     }
@@ -542,13 +595,44 @@ public class SCXMLSerializer {
         if (id != null) {
             b.append(" id=\"").append(id).append("\"");
         }
-        TransitionTarget pt = t.getParent();
-        if (pt != null) {
-            String pid = pt.getId();
-            if (pid != null) {
-                b.append(" parentid=\"").append(pid).append("\"");
+    }
+
+    /**
+     * Serialize namespace declarations for the root SCXML element.
+     *
+     * @param holder The {@link NamespacePrefixesHolder} object
+     * @return The serialized namespace declarations
+     */
+    private static String serializeNamespaceDeclarations(
+            final NamespacePrefixesHolder holder) {
+        Map ns = holder.getNamespaces();
+        StringBuffer b = new StringBuffer();
+        if (ns != null) {
+            Iterator iter = ns.entrySet().iterator();
+            while (iter.hasNext()) {
+                Map.Entry entry = (Map.Entry) iter.next();
+                String prefix = (String) entry.getKey();
+                String nsURI = (String) entry.getValue();
+                if (prefix.length() == 0 && !nsURI.equals(NAMESPACE_SCXML)) {
+                    org.apache.commons.logging.Log log = LogFactory.
+                        getLog(SCXMLSerializer.class);
+                    log.warn("When using the SCXMLSerializer, the default "
+                        + "namespace must be the SCXML namespace:"
+                        + NAMESPACE_SCXML);
+                } if (prefix.equals("cs")
+                        && !nsURI.equals(NAMESPACE_COMMONS_SCXML)) {
+                    org.apache.commons.logging.Log log = LogFactory.
+                        getLog(SCXMLSerializer.class);
+                    log.warn("When using the SCXMLSerializer, the namespace"
+                        + "prefix \"cs\" must bind to the Commons SCXML "
+                        + "namespace:" + NAMESPACE_COMMONS_SCXML);
+                } else if (prefix.length() > 0) {
+                    b.append(" xmlns:").append(prefix).append("=\"").
+                        append(nsURI).append("\"");
+                }
             }
         }
+        return b.toString();
     }
 
     /**
